Stan Math Library: stan/math/prim/scal/prob/binomial_cdf_log.hpp Source File

| Stan Math Library 2.10.0 reverse mode automatic differentiation

- Main Page
- Modules
- Namespaces
- Classes
- Files

- File List
- File Members

binomial_cdf_log.hpp
Go to the documentation of this file.
1 #ifndef STAN_MATH_PRIM_SCAL_PROB_BINOMIAL_CDF_LOG_HPP
2 #define STAN_MATH_PRIM_SCAL_PROB_BINOMIAL_CDF_LOG_HPP
3
4 #include <stan/math/prim/scal/meta/is_constant_struct.hpp>
5 #include <stan/math/prim/scal/meta/partials_return_type.hpp>
6 #include <stan/math/prim/scal/meta/OperandsAndPartials.hpp>
7 #include <stan/math/prim/scal/err/check_consistent_sizes.hpp>
8 #include <stan/math/prim/scal/err/check_bounded.hpp>
9 #include <stan/math/prim/scal/err/check_finite.hpp>
10 #include <stan/math/prim/scal/err/check_greater_or_equal.hpp>
11 #include <stan/math/prim/scal/err/check_less_or_equal.hpp>
12 #include <stan/math/prim/scal/err/check_nonnegative.hpp>
13 #include <stan/math/prim/scal/fun/constants.hpp>
14 #include <stan/math/prim/scal/fun/inv_logit.hpp>
15 #include <stan/math/prim/scal/fun/log1m.hpp>
16 #include <stan/math/prim/scal/fun/log_inv_logit.hpp>
17 #include <stan/math/prim/scal/fun/multiply_log.hpp>
18 #include <stan/math/prim/scal/fun/value_of.hpp>
19 #include <stan/math/prim/scal/fun/binomial_coefficient_log.hpp>
20 #include <stan/math/prim/scal/fun/lbeta.hpp>
21 #include <stan/math/prim/scal/meta/include_summand.hpp>
22 #include <stan/math/prim/scal/fun/inc_beta.hpp>
23 #include <boost/random/binomial_distribution.hpp>
24 #include <boost/random/variate_generator.hpp>
25 #include <cmath>
26
27 namespace stan {
28
29 namespace math {
30
31 template <typename T_n, typename T_N, typename T_prob>
32 typename return_type<T_prob>::type
33 binomial_cdf_log(const T_n& n, const T_N& N, const T_prob& theta) {
34 static const char* function("stan::math::binomial_cdf_log");
35 typedef typename stan::partials_return_type<T_n, T_N, T_prob>::type
36 T_partials_return;
37
38 using stan::math::check_finite;
39 using stan::math::check_bounded;
40 using stan::math::check_nonnegative;
41 using stan::math::value_of;
42 using stan::math::check_consistent_sizes;
43 using stan::math::include_summand;
44
45 // Ensure non-zero arguments lenghts
46 if (!(stan::length(n) && stan::length(N) && stan::length(theta)))
47 return 0.0;
48
49 T_partials_return P(0.0);
50
51 // Validate arguments
52 check_nonnegative(function, "Population size parameter", N);
53 check_finite(function, "Probability parameter", theta);
54 check_bounded(function, "Probability parameter", theta, 0.0, 1.0);
55 check_consistent_sizes(function,
56 "Successes variable", n,
57 "Population size parameter", N,
58 "Probability parameter", theta);
59
60 // Wrap arguments in vector views
61 VectorView<const T_n> n_vec(n);
62 VectorView<const T_N> N_vec(N);
63 VectorView<const T_prob> theta_vec(theta);
64 size_t size = max_size(n, N, theta);
65
66 // Compute vectorized cdf_log and gradient
67 using stan::math::value_of;
68 using stan::math::inc_beta;
69 using stan::math::lbeta;
70 using std::exp;
71 using std::pow;
72 using std::log;
73 using std::exp;
74
75 OperandsAndPartials<T_prob> operands_and_partials(theta);
76
77 // Explicit return for extreme values
78 // The gradients are technically ill-defined,
79 // but treated as negative infinity
80 for (size_t i = 0; i < stan::length(n); i++) {
81 if (value_of(n_vec[i]) < 0)
82 return operands_and_partials.value(stan::math::negative_infinity());
83 }
84
85 for (size_t i = 0; i < size; i++) {
86 // Explicit results for extreme values
87 // The gradients are technically ill-defined, but treated as zero
88 if (value_of(n_vec[i]) >= value_of(N_vec[i])) {
89 continue;
90 }
91 const T_partials_return n_dbl = value_of(n_vec[i]);
92 const T_partials_return N_dbl = value_of(N_vec[i]);
93 const T_partials_return theta_dbl = value_of(theta_vec[i]);
94 const T_partials_return betafunc = exp(lbeta(N_dbl-n_dbl, n_dbl+1));
95 const T_partials_return Pi = inc_beta(N_dbl - n_dbl, n_dbl + 1,
96 1 - theta_dbl);
97
98 P += log(Pi);
99
100 if (!is_constant_struct<T_prob>::value)
101 operands_and_partials.d_x1[i] -= pow(theta_dbl, n_dbl)
102 * pow(1-theta_dbl, N_dbl-n_dbl-1) / betafunc / Pi;
103 }
104
105 return operands_and_partials.value(P);
106 }
107
108 }
109 }
110 #endif
stan
Definition: log_sum_exp.hpp:8
stan::math::value_of
T value_of(const fvar< T > &v)
Return the value of the specified variable.
Definition: value_of.hpp:16
stan::math::lbeta
fvar< T > lbeta(const fvar< T > &x1, const fvar< T > &x2)
Definition: lbeta.hpp:16
stan::math::log
fvar< T > log(const fvar< T > &x)
Definition: log.hpp:15
stan::math::OperandsAndPartials::value
T_return_type value(double value)
Returns a T_return_type with the value specified with the partial derivatves.
Definition: OperandsAndPartials.hpp:68
stan::length
size_t length(const std::vector< T > &x)
Definition: length.hpp:10
stan::math::check_bounded
bool check_bounded(const char *function, const char *name, const T_y &y, const T_low &low, const T_high &high)
Return true if the value is between the low and high values, inclusively.
Definition: check_bounded.hpp:95
binomial_coefficient_log.hpp
is_constant_struct.hpp
stan::math::include_summand
Template metaprogram to calculate whether a summand needs to be included in a proportional (log) prob...
Definition: include_summand.hpp:36
include_summand.hpp
multiply_log.hpp
check_nonnegative.hpp
stan::is_constant_struct
Metaprogram to determine if a type has a base scalar type that can be assigned to type double...
Definition: is_constant_struct.hpp:13
log_inv_logit.hpp
stan::math::inc_beta
fvar< T > inc_beta(const fvar< T > &a, const fvar< T > &b, const fvar< T > &x)
Definition: inc_beta.hpp:20
check_less_or_equal.hpp
stan::math::exp
fvar< T > exp(const fvar< T > &x)
Definition: exp.hpp:10
stan::math::OperandsAndPartials
This class builds partial derivatives with respect to a set of operands.
Definition: OperandsAndPartials.hpp:38
stan::max_size
size_t max_size(const T1 &x1, const T2 &x2)
Definition: max_size.hpp:9
log1m.hpp
partials_return_type.hpp
OperandsAndPartials.hpp
check_bounded.hpp
stan::math::check_finite
bool check_finite(const char *function, const char *name, const T_y &y)
Return true if y is finite.
Definition: check_finite.hpp:62
constants.hpp
stan::math::size
int size(const std::vector< T > &x)
Return the size of the specified standard vector.
Definition: size.hpp:17
stan::math::check_consistent_sizes
bool check_consistent_sizes(const char *function, const char *name1, const T1 &x1, const char *name2, const T2 &x2)
Return true if the dimension of x1 is consistent with x2.
Definition: check_consistent_sizes.hpp:31
check_finite.hpp
lbeta.hpp
inc_beta.hpp
stan::math::pow
fvar< T > pow(const fvar< T > &x1, const fvar< T > &x2)
Definition: pow.hpp:18
stan::math::check_nonnegative
bool check_nonnegative(const char *function, const char *name, const T_y &y)
Return true if y is non-negative.
Definition: check_nonnegative.hpp:66
stan::math::binomial_cdf_log
return_type< T_prob >::type binomial_cdf_log(const T_n &n, const T_N &N, const T_prob &theta)
Definition: binomial_cdf_log.hpp:33
check_greater_or_equal.hpp
value_of.hpp
inv_logit.hpp
stan::VectorView
VectorView is a template expression that is constructed with a container or scalar, which it then allows to be used as an array using operator[].
Definition: VectorView.hpp:48
stan::partials_return_type::type
boost::math::tools::promote_args< typename partials_type< typename scalar_type< T1 >::type >::type, typename partials_type< typename scalar_type< T2 >::type >::type, typename partials_type< typename scalar_type< T3 >::type >::type, typename partials_type< typename scalar_type< T4 >::type >::type, typename partials_type< typename scalar_type< T5 >::type >::type, typename partials_type< typename scalar_type< T6 >::type >::type >::type type
Definition: partials_return_type.hpp:26
check_consistent_sizes.hpp
stan::math::OperandsAndPartials::d_x1
VectorView< T_return_type, false, true > d_x1
Definition: OperandsAndPartials.hpp:39
stan::math::negative_infinity
double negative_infinity()
Return negative infinity.
Definition: constants.hpp:132

[ Stan Home Page ] © 2011–2016, Stan Development Team.